# Copyright (c) 2020 Jerome Arbez-Gindre
# frozen_string_literal: true

require('defmastership')

module DefMastership
  class TotoClassModifier < ModifierBase
  end

  class TutuClassModifier < ModifierBase
  end
end

RSpec.describe(DefMastership::BatchModifier) do
  subject(:batchmodifier) do
    described_class.new(config, adoc_texts)
  end

  let(:config) do
    {
      modifier1: { type: 'toto_class', config: { p: 1 } },
      modifier2: { type: 'toto_class', config: { p: 'whatever1' } },
      modifier3: { type: 'tutu_class', config: { p1: 'whatever2', p2: 'whatever3' } }
    }
  end
  let(:adoc_texts) do
    {
      'file1.adoc' => 'some text',
      'file2.adoc' => 'another text'
    }
  end

  describe '.new' do
    it { is_expected.not_to(be(nil)) }
    it { is_expected.to(have_attributes(config: config)) }

    it do
      expect(batchmodifier).to(have_attributes(adoc_texts: adoc_texts))
    end
  end

  describe '#apply' do
    context 'with only one modification' do
      let(:toto1) { instance_double(DefMastership::TotoClassModifier, 'toto1') }
      let(:adoc_texts_modified) do
        {
          'file1.adoc' => 'some text modified',
          'file2.adoc' => 'another text'
        }
      end

      let(:config_modified) do
        {
          modifier1: { type: 'toto_class', config: { p: 'modified_param' } },
          modifier2: { type: 'toto_class', config: { p: 'whatever1' } },
          modifier3: { type: 'tutu_class', config: { p1: 'whatever2', p2: 'whatever3' } }
        }
      end

      before do
        allow(DefMastership::TotoClassModifier).to(receive(:new).once.and_return(toto1))
        allow(toto1).to(receive(:do_modifications).once.and_return(adoc_texts_modified))
        allow(toto1).to(receive(:config).once.and_return(p: 'modified_param'))
        allow(toto1).to(receive(:changes).once.and_return([%w[from1 to1], %w[from2 to2]]))
        batchmodifier.apply('modifier1')
      end

      it do
        expect(DefMastership::TotoClassModifier).to(have_received(:new).with(p: 1))
      end

      it { expect(toto1).to(have_received(:do_modifications).with(adoc_texts)) }
      it { is_expected.to(have_attributes(adoc_texts: adoc_texts_modified)) }
      it { expect(toto1).to(have_received(:config).with(no_args)) }

      it { is_expected.to(have_attributes(config: config_modified)) }
      it { is_expected.to(have_attributes(changes: [%w[modifier1 from1 to1], %w[modifier1 from2 to2]])) }
    end

    context 'with two modifications' do
      let(:toto2) { instance_double(DefMastership::TotoClassModifier, 'toto2') }
      let(:tutu3) { instance_double(DefMastership::TutuClassModifier, 'tutu3') }

      let(:config_modified) do
        {
          modifier1: { type: 'toto_class', config: { p: 1 } },
          modifier2: { type: 'toto_class', config: :whatever },
          modifier3: { type: 'tutu_class', config: :pouet }
        }
      end

      before do
        allow(DefMastership::TotoClassModifier).to(receive(:new).once.and_return(toto2))
        allow(toto2).to(receive(:do_modifications).once.and_return(:adoc_texts_modified_mod2))
        allow(toto2).to(receive(:config).once.and_return(:whatever))
        allow(DefMastership::TutuClassModifier).to(receive(:new).once.and_return(tutu3))
        allow(tutu3).to(receive(:do_modifications).once.and_return(:adoc_texts_modified_mod3))
        allow(tutu3).to(receive(:config).once.and_return(:pouet))
        allow(toto2).to(receive(:changes).once.and_return([%w[from1 to1]]))
        allow(tutu3).to(receive(:changes).once.and_return([%w[from2 to2]]))
        batchmodifier.apply('modifier2, modifier3')
      end

      it { expect(DefMastership::TotoClassModifier).to(have_received(:new).with(p: 'whatever1')) }
      it { expect(DefMastership::TutuClassModifier).to(have_received(:new).with(p1: 'whatever2', p2: 'whatever3')) }
      it { expect(toto2).to(have_received(:do_modifications).with(adoc_texts)) }
      it { expect(toto2).to(have_received(:config).with(no_args)) }
      it { expect(tutu3).to(have_received(:do_modifications).with(:adoc_texts_modified_mod2)) }
      it { expect(tutu3).to(have_received(:config).with(no_args)) }
      it { is_expected.to(have_attributes(adoc_texts: :adoc_texts_modified_mod3)) }
      it { is_expected.to(have_attributes(config: config_modified)) }
      it { is_expected.to(have_attributes(changes: [%w[modifier2 from1 to1], %w[modifier3 from2 to2]])) }
    end

    context 'with wrong modification' do
      it do
        expect { batchmodifier.apply('wrong-modification') }.to(
          raise_error(ArgumentError, 'wrong-modification is not a known modification')
        )
      end
    end
  end
end
