# Copyright (c) 2020 Jerome Arbez-Gindre
# frozen_string_literal: true

require('defmastership')

RSpec.describe(DefMastership::ChangeRefLineModifier) do
  subject(:refchanger) { described_class.new }

  describe '.new' do
    it { is_expected.not_to(be(nil)) }
    it { is_expected.to(have_attributes(from_regexp: '')) }
    it { is_expected.to(have_attributes(to_template: '')) }
    it { is_expected.to(have_attributes(next_ref:    0)) }
    it { is_expected.to(have_attributes(changes:     [])) }
    it { expect { refchanger.user_defined_attribute }.to(raise_error(NoMethodError)) }
  end

  describe '.from_config' do
    subject(:refchanger) do
      described_class.from_config(
        from_regexp: 'Whatever.+',
        to_template: 'Whatever',
        next_ref: 17
      )
    end

    it { is_expected.not_to(be(nil)) }
    it { is_expected.to(have_attributes(from_regexp: 'Whatever.+')) }
    it { is_expected.to(have_attributes(to_template: 'Whatever')) }
    it { is_expected.to(have_attributes(next_ref:    17)) }
  end

  describe '#from_config' do
    before do
      refchanger.from_config(
        from_regexp: 'Whatever.+',
        to_template: 'Whatever',
        next_ref: 17,
        user_defined_attribute: 'Tadaam'
      )
    end

    it { is_expected.to(have_attributes(from_regexp: 'Whatever.+')) }
    it { is_expected.to(have_attributes(to_template: 'Whatever')) }
    it { is_expected.to(have_attributes(next_ref:    17)) }
    it { is_expected.to(have_attributes(user_defined_attribute: 'Tadaam')) }
  end

  describe '#config' do
    context 'when not initalized' do
      it do
        expect(refchanger.config).to(
          include(from_regexp: '', to_template: '', next_ref: 0)
        )
      end
    end

    context 'when initalized with additionnal keys' do
      before do
        refchanger.from_config(
          next_ref: 1256,
          user_defined_attribute: 'Tadaam'
        )
      end

      it { expect(refchanger.config).to(include(from_regexp: '')) }
      it { expect(refchanger.config).to(include(to_template: '')) }
      it { expect(refchanger.config).to(include(next_ref: 1256)) }
      it { expect(refchanger.config).to(include(user_defined_attribute: 'Tadaam')) }
    end
  end

  describe '#replace_def' do
    context 'when really simple rule' do
      before do
        refchanger.from_config(
          from_regexp: 'TEMP',
          to_template: 'TUTU'
        )
      end

      context 'when valid definition' do
        it do
          expect(refchanger.replace_refdef('[define, whatever, TEMP]'))
            .to(eq('[define, whatever, TUTU]'))
        end
      end

      context 'when no valid definition' do
        it do
          expect(refchanger.replace_refdef('[pouet, whatever, TEMP]'))
            .to(eq('[pouet, whatever, TEMP]'))
        end
      end
    end

    context 'when template is variable' do
      before do
        refchanger.from_config(
          from_regexp: 'TEMP',
          to_template: 'TOTO-%<next_ref>04d',
          next_ref: 124
        )
      end

      it do
        expect(refchanger.replace_refdef('[define, whatever, TEMP]'))
          .to(eq('[define, whatever, TOTO-0124]'))
      end

      it do
        refchanger.replace_refdef('[define, whatever, TEMP]')
        expect(refchanger).to(have_attributes(next_ref: 125))
      end
    end

    context 'when capture group in regexp' do
      before do
        refchanger.from_config(
          from_regexp: '(?<cg>TOTO|TITI)-TEMP[\d]*',
          to_template: '%<cg>s-%<next_ref>04d',
          next_ref: 132
        )
      end

      it do
        expect(refchanger.replace_refdef('[define, whatever, TOTO-TEMP]'))
          .to(eq('[define, whatever, TOTO-0132]'))
      end

      it do
        refchanger.replace_refdef('[define, whatever, TOTO-TEMP]')
        expect(refchanger.replace_refdef('[define, whatever, TITI-TEMP]'))
          .to(eq('[define, whatever, TITI-0133]'))
      end

      it do
        refchanger.replace_refdef('[define, whatever, TOTO-TEMP1]')
        refchanger.replace_refdef('[define, whatever, TITI-TEMP2]')
        expect(refchanger).to(have_attributes(changes: [%w[TOTO-TEMP1 TOTO-0132], %w[TITI-TEMP2 TITI-0133]]))
      end
    end

    context 'when definition is in literal block' do
      before do
        refchanger.from_config(
          from_regexp: 'TEMP',
          to_template: 'TUTU'
        )
        refchanger.replace_refdef('....')
      end

      it do
        expect(refchanger.replace_refdef('[define, whatever, TEMP]'))
          .to(eq('[define, whatever, TEMP]'))
      end
    end

    context 'when defintion is after literal block' do
      before do
        refchanger.from_config(
          from_regexp: 'TEMP',
          to_template: 'TUTU'
        )
        refchanger.replace_refdef("....\n")
        refchanger.replace_refdef('[define, whatever, TEMP]')
        refchanger.replace_refdef("....\n")
      end

      it do
        expect(refchanger.replace_refdef('[define, whatever, TEMP]'))
          .to(eq('[define, whatever, TUTU]'))
      end
    end
  end

  describe '#replace_irefs' do
    before do
      refchanger.from_config(
        from_regexp: '(?<cg>TOTO|TITI)-TEMP[\d]*',
        to_template: '%<cg>s-%<next_ref>04d',
        next_ref: 132
      )
      refchanger.replace_refdef('[define, whatever, TOTO-TEMP123]')
      refchanger.replace_refdef('[define, whatever, TITI-TEMP421]')
    end

    it do
      expect(refchanger.replace_irefs('defs:iref[TOTO-TEMP1234]'))
        .to(eq('defs:iref[TOTO-TEMP1234]'))
    end

    it do
      expect(refchanger.replace_irefs('defs:iref[TOTO-TEMP123]'))
        .to(eq('defs:iref[TOTO-0132]'))
    end

    it do
      expect(
        refchanger.replace_irefs(
          'defs:iref[TOTO-TEMP123] defs:iref[TITI-TEMP421] bla'
        )
      ).to(eq('defs:iref[TOTO-0132] defs:iref[TITI-0133] bla'))
    end
  end

  describe '#replace' do
    before do
      refchanger.from_config(
        from_regexp: 'TEMP',
        to_template: 'TUTU'
      )
    end

    it do
      expect(refchanger.replace(:refdef, '[define, whatever, TEMP]'))
        .to(eq('[define, whatever, TUTU]'))
    end

    it do
      expect(refchanger.replace(:refdef, '[define, whatever, TEMP(a~12345678)]'))
        .to(eq('[define, whatever, TUTU(a~12345678)]'))
    end

    it do
      expect(refchanger.replace(:refdef, '[define, whatever, TEMP(~12345678)]'))
        .to(eq('[define, whatever, TUTU(~12345678)]'))
    end

    it do
      expect(refchanger.replace(:refdef, '[define, whatever, TEMP(a)]'))
        .to(eq('[define, whatever, TUTU(a)]'))
    end

    it do
      refchanger.replace(:refdef, '[define, whatever, TEMP]')
      expect(refchanger.replace(:irefs, 'defs:iref[TEMP]'))
        .to(eq('defs:iref[TUTU]'))
    end

    it { expect { refchanger.replace(:pouet, 'whatever') }.to(raise_error(NoMethodError)) }
  end
end
