# Copyright (c) 2020 Jerome Arbez-Gindre
# frozen_string_literal: true

require('defmastership')

RSpec.describe(DefMastership::CSVFormatterBody) do
  subject(:formatter) { described_class.new(document) }

  let(:document)   { instance_double(DefMastership::Document, 'document')     }
  let(:definition) { instance_double(DefMastership::Definition, 'definition') }

  describe '.new' do
    it { is_expected.not_to(be(nil)) }
  end

  describe '#fixed' do
    before do
      allow(definition).to(receive(:type).with(no_args).and_return('a'))
      allow(definition).to(receive(:reference).with(no_args).and_return('b'))
      allow(definition).to(receive(:value).with(no_args).and_return('c'))
      allow(definition).to(receive(:sha256).with(no_args).and_return('d'))
      formatter.fixed(definition)
    end

    it { expect(definition).to(have_received(:type).with(no_args)) }
    it { expect(definition).to(have_received(:reference).with(no_args)) }
    it { expect(definition).to(have_received(:value).with(no_args)) }
    it { expect(definition).to(have_received(:sha256).with(no_args)) }
    it { expect(formatter.fixed(definition)).to(eq(%w[a b c d])) }
  end

  describe '#wrong_explicit_checksum' do
    context 'when no wrong_explicit checksum' do
      before do
        allow(definition).to(receive(:wrong_explicit_checksum).with(no_args).and_return(nil))
        formatter.wrong_explicit_checksum(definition)
      end

      it { expect(definition).to(have_received(:wrong_explicit_checksum).with(no_args)) }
      it { expect(formatter.wrong_explicit_checksum(definition)).to(eq([''])) }
    end

    context 'when explicit checksum' do
      before do
        allow(definition).to(receive(:wrong_explicit_checksum).with(no_args).and_return('ab12'))
        formatter.wrong_explicit_checksum(definition)
      end

      it { expect(formatter.wrong_explicit_checksum(definition)).to(eq(['ab12'])) }
    end
  end

  describe '#explicit_version' do
    context 'when no explicit_version' do
      before do
        allow(definition).to(receive(:explicit_version).with(no_args).and_return(nil))
        formatter.explicit_version(definition)
      end

      it { expect(definition).to(have_received(:explicit_version).with(no_args)) }
      it { expect(formatter.explicit_version(definition)).to(eq([''])) }
    end

    context 'when explicit_version' do
      before do
        allow(definition).to(receive(:explicit_version).with(no_args).and_return('ab12'))
        formatter.explicit_version(definition)
      end

      it { expect(formatter.explicit_version(definition)).to(eq(['ab12'])) }
    end
  end

  describe '#labels' do
    before { allow(document).to(receive(:labels).with(no_args).and_return(['whatever'])) }

    context 'when no labels on definition' do
      before do
        allow(definition).to(receive(:labels).with(no_args).and_return([]))
        formatter.labels(definition)
      end

      it { expect(definition).to(have_received(:labels).once.with(no_args)) }
      it { expect(formatter.labels(definition)).to(eq([''])) }
    end

    context 'when labels on definition' do
      before do
        allow(definition).to(receive(:labels).with(no_args).and_return(%w[toto tutu]))
        formatter.labels(definition)
      end

      it { expect(definition).to(have_received(:labels).once.with(no_args)) }
      it { expect(formatter.labels(definition)).to(eq(["toto\ntutu"])) }
    end
  end

  describe '#eref' do
    context 'when no eref on the document' do
      before do
        allow(document).to(receive(:eref).with(no_args).and_return({}))
        formatter.eref(definition)
      end

      it { expect(document).to(have_received(:eref).with(no_args)) }
      it { expect(formatter.eref(nil)).to(eq([])) }
    end

    context 'when eref on the document' do
      before do
        allow(document).to(
          receive(:eref).with(no_args).and_return(
            a: 'whatever', b: 'whatever', c: 'whatever'
          )
        )
        allow(definition).to(receive(:eref).with(no_args).and_return(a: %w[A B], b: [], c: ['C']))
        formatter.eref(definition)
      end

      it { expect(definition).to(have_received(:eref).exactly(3).times.with(no_args)) }
      it { expect(formatter.eref(definition)).to(eq(["A\nB", '', 'C'])) }
    end
  end

  describe '#iref' do
    before { allow(document).to(receive(:iref).with(no_args).and_return(true)) }

    context 'when no iref on the definition' do
      before do
        allow(definition).to(receive(:iref).with(no_args).and_return([]))
        formatter.iref(definition)
      end

      it { expect(definition).to(have_received(:iref).with(no_args)) }
      it { expect(formatter.iref(definition)).to(eq([''])) }
    end

    context 'when iref on the definition' do
      before do
        allow(definition).to(receive(:iref).with(no_args).and_return(%w[A B]))
        formatter.iref(definition)
      end

      it { expect(definition).to(have_received(:iref).with(no_args)) }
      it { expect(formatter.iref(definition)).to(eq(["A\nB"])) }
    end
  end

  describe '#attributes' do
    context 'when no attributes on the document' do
      before do
        allow(document).to(receive(:attributes).with(no_args).and_return({}))
        formatter.attributes(definition)
      end

      it { expect(document).to(have_received(:attributes).with(no_args)) }
      it { expect(formatter.attributes(nil)).to(eq([])) }
    end

    context 'when attributes on the document' do
      before do
        allow(document).to(
          receive(:attributes).with(no_args)
            .and_return(a: 'whatever', b: 'whatever', c: 'whatever')
        )
        allow(definition).to(receive(:attributes).and_return(a: 'X', b: '', c: 'Y'))
        formatter.attributes(definition)
      end

      it { expect(definition).to(have_received(:attributes).exactly(3).times) }
      it { expect(formatter.attributes(definition)).to(eq(['X', '', 'Y'])) }
    end
  end
end
