# Copyright (c) 2020 Jerome Arbez-Gindre
# frozen_string_literal: true

require('defmastership')
require('ostruct')
require('csv')

RSpec.describe(DefMastership::CSVFormatter) do
  subject(:formatter) { described_class.new(document, ';') }

  let(:document) { instance_double(DefMastership::Document, 'document') }

  describe '.new' do
    it { is_expected.not_to(be(nil)) }
  end

  describe '#export_to' do
    let(:header) { instance_double(DefMastership::CSVFormatterHeader, 'header') }
    let(:body) { instance_double(DefMastership::CSVFormatterBody, 'body') }
    let(:csv)  { instance_double(CSV, 'csv')                              }

    before do
      allow(CSV).to(receive(:open).and_yield(csv))
      allow(csv).to(receive(:<<))
      allow(DefMastership::CSVFormatterHeader).to(receive(:new).with(document).and_return(header))
      allow(DefMastership::CSVFormatterBody).to(receive(:new).with(document).and_return(body))
      allow(document).to(receive(:definitions).and_return(%i[def1 def2]))
      allow(document).to(receive(:wrong_explicit_checksum?).with(no_args).and_return(false))
      allow(document).to(receive(:explicit_version?).with(no_args).and_return(false))
      allow(document).to(receive(:labels).with(no_args).and_return([]))
      allow(document).to(receive(:eref).with(no_args).and_return([]))
      allow(document).to(receive(:iref).with(no_args).and_return(false))
      allow(document).to(receive(:attributes).with(no_args).and_return([]))
    end

    context 'when no variable columns' do
      methods = %i[fixed]

      before do
        methods.each do |method|
          allow(header).to(receive(method).with(no_args).and_return(["#{method} header"]))
          allow(body).to(receive(method).with(:def1).and_return(["#{method} def1 body"]))
          allow(body).to(receive(method).with(:def2).and_return(["#{method} def2 body"]))
        end
        formatter.export_to('whatever')
      end

      methods.each do |method|
        it { expect(header).to(have_received(method).with(no_args)) }
        it { expect(body).to(have_received(method).with(:def1)) }
        it { expect(body).to(have_received(method).with(:def2)) }
      end

      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} header" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def1 body" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def2 body" })) }
    end

    context 'when wrong_explicit_checksum' do
      methods = %i[fixed wrong_explicit_checksum]

      before do
        methods.each do |method|
          allow(header).to(receive(method).with(no_args).and_return(["#{method} header"]))
          allow(body).to(receive(method).with(:def1).and_return(["#{method} def1 body"]))
          allow(body).to(receive(method).with(:def2).and_return(["#{method} def2 body"]))
        end
        allow(document).to(receive(:wrong_explicit_checksum?).with(no_args).and_return(true))
        formatter.export_to('whatever')
      end

      methods.each do |method|
        it { expect(header).to(have_received(method).with(no_args)) }
        it { expect(body).to(have_received(method).with(:def1)) }
        it { expect(body).to(have_received(method).with(:def2)) }
      end

      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} header" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def1 body" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def2 body" })) }
    end

    context 'when explicit_version' do
      methods = %i[fixed explicit_version]

      before do
        methods.each do |method|
          allow(header).to(receive(method).with(no_args).and_return(["#{method} header"]))
          allow(body).to(receive(method).with(:def1).and_return(["#{method} def1 body"]))
          allow(body).to(receive(method).with(:def2).and_return(["#{method} def2 body"]))
        end
        allow(document).to(receive(:explicit_version?).with(no_args).and_return(true))
        formatter.export_to('whatever')
      end

      methods.each do |method|
        it { expect(header).to(have_received(method).with(no_args)) }
        it { expect(body).to(have_received(method).with(:def1)) }
        it { expect(body).to(have_received(method).with(:def2)) }
      end

      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} header" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def1 body" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def2 body" })) }
    end

    context 'when labels' do
      methods = %i[fixed labels]

      before do
        methods.each do |method|
          allow(header).to(receive(method).with(no_args).and_return(["#{method} header"]))
          allow(body).to(receive(method).with(:def1).and_return(["#{method} def1 body"]))
          allow(body).to(receive(method).with(:def2).and_return(["#{method} def2 body"]))
        end
        allow(document).to(receive(:labels).with(no_args).and_return([:whatever]))
        formatter.export_to('whatever')
      end

      methods.each do |method|
        it { expect(header).to(have_received(method).with(no_args)) }
        it { expect(body).to(have_received(method).with(:def1)) }
        it { expect(body).to(have_received(method).with(:def2)) }
      end

      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} header" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def1 body" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def2 body" })) }
    end

    context 'when eref' do
      methods = %i[fixed eref]

      before do
        methods.each do |method|
          allow(header).to(receive(method).with(no_args).and_return(["#{method} header"]))
          allow(body).to(receive(method).with(:def1).and_return(["#{method} def1 body"]))
          allow(body).to(receive(method).with(:def2).and_return(["#{method} def2 body"]))
        end
        allow(document).to(receive(:eref).with(no_args).and_return([:whatever]))
        formatter.export_to('whatever')
      end

      methods.each do |method|
        it { expect(header).to(have_received(method).with(no_args)) }
        it { expect(body).to(have_received(method).with(:def1)) }
        it { expect(body).to(have_received(method).with(:def2)) }
      end

      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} header" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def1 body" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def2 body" })) }
    end

    context 'when iref' do
      methods = %i[fixed iref]

      before do
        methods.each do |method|
          allow(header).to(receive(method).with(no_args).and_return(["#{method} header"]))
          allow(body).to(receive(method).with(:def1).and_return(["#{method} def1 body"]))
          allow(body).to(receive(method).with(:def2).and_return(["#{method} def2 body"]))
        end
        allow(document).to(receive(:iref).with(no_args).and_return(true))
        formatter.export_to('whatever')
      end

      methods.each do |method|
        it { expect(header).to(have_received(method).with(no_args)) }
        it { expect(body).to(have_received(method).with(:def1)) }
        it { expect(body).to(have_received(method).with(:def2)) }
      end

      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} header" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def1 body" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def2 body" })) }
    end

    context 'when attributes' do
      methods = %i[fixed attributes]

      before do
        methods.each do |method|
          allow(header).to(receive(method).with(no_args).and_return(["#{method} header"]))
          allow(body).to(receive(method).with(:def1).and_return(["#{method} def1 body"]))
          allow(body).to(receive(method).with(:def2).and_return(["#{method} def2 body"]))
        end
        allow(document).to(receive(:attributes).with(no_args).and_return([:whatever]))
        formatter.export_to('whatever')
      end

      methods.each do |method|
        it { expect(header).to(have_received(method).with(no_args)) }
        it { expect(body).to(have_received(method).with(:def1)) }
        it { expect(body).to(have_received(method).with(:def2)) }
      end

      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} header" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def1 body" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def2 body" })) }
    end

    context 'when every colums' do
      methods = %i[fixed wrong_explicit_checksum explicit_version labels eref iref attributes]

      before do
        methods.each do |method|
          allow(header).to(receive(method).with(no_args).and_return(["#{method} header"]))
          allow(body).to(receive(method).with(:def1).and_return(["#{method} def1 body"]))
          allow(body).to(receive(method).with(:def2).and_return(["#{method} def2 body"]))
        end
        allow(document).to(receive(:wrong_explicit_checksum?).with(no_args).and_return(true))
        allow(document).to(receive(:explicit_version?).with(no_args).and_return(true))
        allow(document).to(receive(:labels).with(no_args).and_return([:whatever]))
        allow(document).to(receive(:eref).with(no_args).and_return([:whatever]))
        allow(document).to(receive(:iref).with(no_args).and_return(true))
        allow(document).to(receive(:attributes).with(no_args).and_return([:whatever]))
        formatter.export_to('whatever')
      end

      methods.each do |method|
        it { expect(header).to(have_received(method).with(no_args)) }
        it { expect(body).to(have_received(method).with(:def1)) }
        it { expect(body).to(have_received(method).with(:def2)) }
      end

      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} header" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def1 body" })) }
      it { expect(csv).to(have_received(:<<).with(methods.map { |method| "#{method} def2 body" })) }
    end
  end

  context 'when #export_to csv file' do
    let(:target_file) { 'export.csv' }
    let(:definitions) do
      [
        OpenStruct.new(type: 'a', reference: 'b', value: 'c', sha256: 'd'),
        OpenStruct.new(type: 'd', reference: 'e', value: 'f', sha256: 'g')
      ]
    end

    before do
      setup_aruba
      allow(document).to(receive(:labels).with(no_args).and_return([]))
      allow(document).to(receive(:eref).with(no_args).and_return({}))
      allow(document).to(receive(:iref).with(no_args).and_return(false))
      allow(document).to(receive(:attributes).with(no_args).and_return({}))
      allow(document).to(receive(:definitions).with(no_args).and_return(definitions))
      allow(document).to(receive(:wrong_explicit_checksum?).with(no_args).and_return(false))
      allow(document).to(receive(:explicit_version?).with(no_args).and_return(false))
      formatter.export_to("#{aruba.current_directory}/#{target_file}")
    end

    it { expect(document).to(have_received(:labels).with(no_args)) }
    it { expect(document).to(have_received(:eref).with(no_args)) }
    it { expect(document).to(have_received(:iref).with(no_args)) }
    it { expect(document).to(have_received(:attributes).with(no_args)) }
    it { expect(document).to(have_received(:definitions).with(no_args)) }
    it { expect(document).to(have_received(:wrong_explicit_checksum?).with(no_args)) }
    it { expect(document).to(have_received(:explicit_version?).with(no_args)) }

    it do
      expect(target_file).to(have_file_content(<<~CSV_FILE))
        Type;Reference;Value;Checksum
        a;b;c;d
        d;e;f;g
      CSV_FILE
    end
  end
end
